DEFAULT_ASSIGNMENT_POINTS = 1
TASK_TYPE_SYNC_CANVAS_ENROLLMENTS = "sync_canvas_enrollments"
TASK_TYPE_PUSH_EDX_GRADES_TO_CANVAS = "push_edx_grades_to_canvas"
CANVAS_TASK_TYPES = [
    TASK_TYPE_SYNC_CANVAS_ENROLLMENTS,
    TASK_TYPE_PUSH_EDX_GRADES_TO_CANVAS,
]
COURSE_KEY_ID_EMPTY = "Course Key/ID is empty."
